// con lo shading e con lo zbuffer 

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>	
#include <GL/glu.h>	
#include <stdlib.h>
#include <time.h>
#include <SDL.h>

void DrawSolidBlock(float xsz, float ysz, float zsz)
{
	glBegin(GL_QUAD_STRIP);
	glNormal3f(.0f,.0f,-1.0f);
	glVertex3f(-xsz, ysz, zsz);
  glVertex3f(-xsz, -ysz, zsz);
	glVertex3f(xsz, ysz, zsz);
  glVertex3f(xsz, -ysz, zsz);
	
	
	glNormal3f(1.0f,.0f,.0f);
	glVertex3f(xsz, ysz, -zsz);
	glVertex3f(xsz, -ysz, -zsz);
  glNormal3f(.0f,.0f,-1.0f);
	glVertex3f(-xsz, ysz, -zsz);
	glVertex3f(-xsz, -ysz, -zsz);
	glNormal3f(-1.0f,.0f,.0f);
	glVertex3f(-xsz, ysz, zsz);
	glVertex3f(-xsz, -ysz, zsz);
	glEnd();

  glBegin(GL_QUADS);
	glNormal3f(.0f,1.0f,.0f);
	glVertex3f(-xsz, ysz, zsz);
	glVertex3f(xsz, ysz, zsz);
	glVertex3f(xsz, ysz, -zsz);
  glVertex3f(-xsz, ysz, -zsz);

	glNormal3f(.0f,-1.0f,.0f);
	glVertex3f(-xsz, -ysz, -zsz);
  glVertex3f(xsz, -ysz, -zsz);
	glVertex3f(xsz, -ysz, zsz);
  glVertex3f(-xsz, -ysz, zsz);
  glEnd();

}

void DrawDoubleBlock(float xsz, float ysz, float zsz, float distance)
{
	glPushMatrix();
	glTranslatef(0,distance,0);
	DrawSolidBlock(xsz,ysz,zsz);
	glTranslatef(0,-2*distance,0);
	DrawSolidBlock(xsz,ysz,zsz);
	glPopMatrix();
}

void DrawTranslatedRollingBall(float radius,float angle)
{
	const float MysteriousConstant = -3;
	glPushMatrix();
	glTranslatef(0,radius*1.5,0);
	glRotatef(angle*MysteriousConstant,1,0,0);
	DrawSolidBlock(radius,radius,radius);
	glPopMatrix();
}


void DrawMoebiusRollingBall(float r, int twist, float angle, float radius)
{
	float TwistPerDeg=(twist*180.0)/360.0f;

  	glPushMatrix();
		glRotatef(angle,0,1,0);
		glTranslatef(r,0,0);
		glRotatef(angle*TwistPerDeg,0,0,1);
		DrawTranslatedRollingBall(radius,angle);
		glPopMatrix();
}

void DrawMoebiusRing(int step, float r, int twist)
{
	float angle=360.0f/step;
	float angletwist=(180.0f*twist)/step;
	for(int i=0;i<step;++i)
	{
		glPushMatrix();
		glRotatef(i*angle,0,1,0);
		glTranslatef(r,0,0);
		glRotatef(i*angletwist,0,0,1);
		DrawSolidBlock(1.5,.2,.5);
		glPopMatrix();
	}
}

void DrawGLScene()
{
	static int LastT=clock();
	static float CurAngle=0;
  int CurT=clock();
	float DeltaT=(CurT-LastT)/1000.0;  // elapsed time in secs.
	if(DeltaT < 0.01) return;
	LastT=CurT;
	float AngularSpeed = 90;  // in degree/sec
	CurAngle+=AngularSpeed*DeltaT;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	gluLookAt(2,5,12,0,0,0,0,1,0);

	glRotatef(-CurAngle,0,1,0);
	DrawMoebiusRing(18,5,1);
	DrawMoebiusRollingBall(5,1,CurAngle,.5);
  SDL_GL_SwapBuffers();
}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
}

int main(int argc, char **argv) 
{  
  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);   
  
	/* Create a 640x480 OpenGL screen */
  if ( SDL_SetVideoMode(640, 480, 0, SDL_OPENGL |SDL_RESIZABLE) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

	/* Loop, drawing and checking events */
	int done = 0;
	int ret;
  myGLReshapeFunc(640,480);  
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glShadeModel(GL_FLAT );
	glEnable(GL_DEPTH_TEST);
	while ( ! done ) 
	{
    SDL_Event event;
	  ret=SDL_PollEvent(&event);
		//printf(".");fflush(stdout);
		if(ret)
			switch(event.type)
			{  
						case SDL_QUIT    : done = 1;  
															break ;
						case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
																				done = 1;
															break;
						case SDL_VIDEORESIZE	: SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL |SDL_RESIZABLE);
																		myGLReshapeFunc(event.resize.w,event.resize.h); 
																		break;
			}  
		else DrawGLScene();
  }
  SDL_Quit();
  return 1;
}
